/************************************************************************
 ***   GROOVESIZER Tutorial MB001 - Controlling the LED grid
 ***   for the GROOVESIZER
 ***   http://groovesizer.com 
 ************************************************************************
 * Copyright (C) 2013 MoShang (Jean Marais) moshang@groovesizer.com
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/
 * 
 ************************************************************************/

// the three pins needed to talk to the 74HC595 shift registers
// as for the Alpha firmware jumper setup
const byte LEDlatchPin = 2;
const byte LEDclockPin = 3; // change the 3 to a 6 for the Delta firmware jumper setup
const byte LEDdataPin = 4;

byte LEDrow[5]; // an array of 5 bytes - each byte holds the on/off states for one of the LED rows (8 bits to a byte - 8 LEDs per row)

void setup()
{
  // set the pin mode for the shift register control pins
  pinMode(LEDlatchPin, OUTPUT);
  pinMode(LEDclockPin, OUTPUT);
  pinMode(LEDdataPin, OUTPUT);
}

void loop()
{
  clearLEDs(); // a function to clear the temporary LEDrow bytes
  
  // lets define the pattern we want to send to the grid
  // if a bit is 1, the LED is on
  // if a bit is 0, the LED is off
  
  LEDrow[0] = B10000001;
  LEDrow[1] = B01000010;
  LEDrow[2] = B00100100;
  LEDrow[3] = B00011000;
  LEDrow[4] = B10011001;
  
  /*
  /////////
  *      *
   *    * 
    *  *  
     **   
  *  **  *
  ////////
  */
  
  updateLEDs(); // a function that updates the LEDs by shifting the LEDrow bytes out to the 74HC595s 
}

void clearLEDs()
{
  for (byte i = 0; i < 5; i++) // if there's only one line in the for-loop, we don't have to use curly braces {}
    LEDrow[i] = 0; // 0 in this case is the same as B00000000, ie. all bits/LEDs off
}

void updateLEDs()
{
  // we can't shift out all 5 bytes at once
  // if we were to send all 5 at once and all 40 of the LEDs were on, we'd draw so much current that we'd starve the processor (trust me, I've done this)
  // instead, we alternate between sending the top 2 rows (and the bottom 3 off), and the bottom 3 rows (with the top 2 off)
  // this way at most 24 LEDs can be on at once
  // we'll need a variable to keep track of which rows we sent last
  static boolean lastSentTop = false; 

  digitalWrite(LEDlatchPin, LOW); // we ground the LEDlatchPin and hold it low for as long as we are shifting bytes out

  if (!lastSentTop) // send the top 2 rows - also keep in mind that we need to send the rows in reverse (sending the bottom row first and the top row last)
  {
    shiftOut(LEDdataPin, LEDclockPin, LSBFIRST, B00000000); // row 5 for the sake of clarity, I'm not using a for-loop here
    shiftOut(LEDdataPin, LEDclockPin, LSBFIRST, B00000000); // row 4
    shiftOut(LEDdataPin, LEDclockPin, LSBFIRST, B00000000); // row 3
    shiftOut(LEDdataPin, LEDclockPin, LSBFIRST, LEDrow[1]); // row 2 
    shiftOut(LEDdataPin, LEDclockPin, LSBFIRST, LEDrow[0]); // row 1
    lastSentTop = true;
  }
  else
  {
    shiftOut(LEDdataPin, LEDclockPin, LSBFIRST, LEDrow[4]);
    shiftOut(LEDdataPin, LEDclockPin, LSBFIRST, LEDrow[3]); 
    shiftOut(LEDdataPin, LEDclockPin, LSBFIRST, LEDrow[2]);
    shiftOut(LEDdataPin, LEDclockPin, LSBFIRST, B00000000); 
    shiftOut(LEDdataPin, LEDclockPin, LSBFIRST, B00000000);
    lastSentTop = false;
  }
  digitalWrite(LEDlatchPin, HIGH); //we return the latch pin high to stop shifting
}





